Package: ExCluster
Title: ExCluster robustly detects differentially expressed exons
        between two conditions of RNA-seq data, requiring at least two
        independent biological replicates per condition
Version: 1.29.0
Author: R. Matthew Tanner, William L. Stanford, and Theodore J. Perkins
Date: 2018-10-02
Maintainer: R. Matthew Tanner <rtann038@uottawa.ca>
Description: ExCluster flattens Ensembl and GENCODE GTF files into GFF files,
    which are used to count reads per non-overlapping exon bin from BAM files.
    This read counting is done using the function featureCounts from the package
    Rsubread. Library sizes are normalized across all biological replicates, and
    ExCluster then compares two different conditions to detect signifcantly 
    differentially spliced genes. This process requires at least two independent
    biological repliates per condition, and ExCluster accepts only exactly two
    conditions at a time. ExCluster ultimately produces false discovery rates
    (FDRs) per gene, which are used to detect significance. Exon log2 fold
    change (log2FC) means and variances may be plotted for each significantly 
    differentially spliced gene, which helps scientists develop hypothesis and
    target differential splicing events for RT-qPCR validation in the wet lab.
biocViews: ImmunoOncology, DifferentialSplicing, RNASeq, Software
Imports: stats, methods, grDevices, graphics, utils
Depends: Rsubread, GenomicRanges, rtracklayer, matrixStats, IRanges
License: GPL-3
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/ExCluster
git_branch: devel
git_last_commit: 9571f20
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 22:30:37 UTC; biocbuild
