export('GFF_convert')
export('processCounts')
export('ExCluster')
export('plotExonlog2FC')
export('GRangesFromExClustResults')

# import packages
import('stats')
import('methods')
import('grDevices')
import('graphics')
import('utils')
import('matrixStats')
importFrom('IRanges','IRanges')
import('Rsubread')
import('GenomicRanges')
importFrom('rtracklayer','import')
importFrom('rtracklayer','export')
